###   Efficiency Spillovers in Bayesian Stochastic Frontier Models: ###
###     Application to electricity distribution in New Zealand      ###

## December 2017
## As submitted to Spatial Economic Analysis
## NOTE: This code uses other R packages and code from other authors.

## Note the parameters methodrho and methodu.
## Method 1 is Random Walk Metropolis with a normal candidate distribution
## Method 2 is a Guided Walk Metropolis with a normal candidate with a normal candidate distribution.
## Method 3 is Slice Sampling with original code from Radford Neal (Neal, 2002)
## Methods 1, 2 and 3 available for methodrho (Spatial Parameter)
## Methods 1 and 2 available for methodu (Efficiencies)
## For syntax and an example, see the code at the very end of the script

### LOAD NECESSARY PACKAGES ###

library(Matrix)
library(MASS)
library(ks)
library(numDeriv)
library(truncnorm)
library(mvtnorm)
library(geoR)
library(dlm)
library(spdep) ;  library(reshape)
library(plyr)
library(coda)
library(robustbase)

#### Function for Slice Sampling by Radford Neal ####

# R FUNCTIONS FOR PERFORMING UNIVARIATE SLICE SAMPLING.
#
# Radford M. Neal, 17 March 2008.
#
# Implements, with slight modifications and extensions, the algorithm described
# in Figures 3 and 5 of the following paper:
#
#   Neal, R. M (2003) "Slice sampling" (with discussion), Annals of Statistics,
#      vol. 31, no. 3, pp. 705-767.
#
# See the documentation for the function uni.slice below for how to use it.
# The function uni.slice.test was used to test the uni.slice function.


# GLOBAL VARIABLES FOR RECORDING PERFORMANCE.

uni.slice.calls <- 0  # Number of calls of the slice sampling function
uni.slice.evals <- 0	# Number of density evaluations done in these calls


# UNIVARIATE SLICE SAMPLING WITH STEPPING OUT AND SHRINKAGE.
#
# Performs a slice sampling update from an initial point to a new point that
# leaves invariant the distribution with the specified log density function.
#
# Arguments:
#
#   x0    Initial point
#   g     Function returning the log of the probability density (plus constant)
#   w     Size of the steps for creating interval (default 1)
#   m     Limit on steps (default infinite)
#   lower Lower bound on support of the distribution (default -Inf)
#   upper Upper bound on support of the distribution (default +Inf)
#   gx0   Value of g(x0), if known (default is not known)
#
# The log density function may return -Inf for points outside the support
# of the distribution.  If a lower and/or upper bound is specified for the
# support, the log density function will not be called outside such limits.
#
# The value of this function is the new point sampled, with an attribute
# of "log.density" giving the value of the log density function, g, at this
# point.  Depending on the context, this log density might be passed as the
# gx0 argument of a future call of uni.slice.
#
# The global variable uni.slice.calls is incremented by one for each call
# of uni.slice.  The global variable uni.slice.evals is incremented by the
# number of calls made to the g function passed.
#
# WARNING:  If you provide a value for g(x0), it must of course be correct!
# In addition to giving wrong answers, wrong values for gx0 may result in
# the uni.slice function going into an infinite loop.

uni.slice <- function (x0, g, w=NULL, m=Inf, lower=NULL, upper=NULL, gx0=NULL)
{
  # Check the validity of the arguments.

  if (!is.numeric(x0) || length(x0)!=1
      || !is.function(g)
      || !is.numeric(w) || length(w)!=1 || w<=0
      || !is.numeric(m) || !is.infinite(m) && (m<=0 || m>1e9 || floor(m)!=m)
      || !is.numeric(lower) || length(lower)!=1 || x0<lower
      || !is.numeric(upper) || length(upper)!=1 || x0>upper
      || upper<=lower
      || !is.null(gx0) && (!is.numeric(gx0) || length(gx0)!=1))
  {
    stop ("Invalid slice sampling argument")
  }

  # Keep track of the number of calls made to this function.

  uni.slice.calls <<- uni.slice.calls + 1

  # Find the log density at the initial point, if not already known.

  if (is.null(gx0))
  { uni.slice.evals <<- uni.slice.evals + 1
    gx0 <- g(x0)
  }

  # Determine the slice level, in log terms.

  logy <- gx0 - rexp(1)

  # Find the initial interval to sample from.

  u <- runif(1,0,w)
  L <- x0 - u
  R <- x0 + (w-u)  # should guarantee that x0 is in [L,R], even with roundoff

  # Expand the interval until its ends are outside the slice, or until
  # the limit on steps is reached.

  if (is.infinite(m))  # no limit on number of steps
  {
    repeat
    { if (L<=lower) break
      uni.slice.evals <<- uni.slice.evals + 1
      if (g(L)<=logy) break
      L <- L - w
    }

    repeat
    { if (R>=upper) break
      uni.slice.evals <<- uni.slice.evals + 1
      if (g(R)<=logy) break
      R <- R + w
    }
  }

  else if (m>1)  # limit on steps, bigger than one
  {
    J <- floor(runif(1,0,m))
    K <- (m-1) - J

    while (J>0)
    { if (L<=lower) break
      uni.slice.evals <<- uni.slice.evals + 1
      if (g(L)<=logy) break
      L <- L - w
      J <- J - 1
    }

    while (K>0)
    { if (R>=upper) break
      uni.slice.evals <<- uni.slice.evals + 1
      if (g(R)<=logy) break
      R <- R + w
      K <- K - 1
    }
  }

  # Shrink interval to lower and upper bounds.

  if (L<lower)
  { L <- lower
  }
  if (R>upper)
  { R <- upper
  }

  # Sample from the interval, shrinking it on each rejection.

  repeat
  {
    x1 <- runif(1,L,R)

    uni.slice.evals <<- uni.slice.evals + 1
    gx1 <- g(x1)

    if (gx1>=logy) break

    if (x1>x0)
    { R <- x1
    }
    else
    { L <- x1
    }
  }

  # Return the point sampled, with its log density attached as an attribute.

  attr(x1,"log.density") <- gx1
  return (x1)

}


### Own code by Author begins ###

Simulation <- function(n.repetitions=NULL, J=NULL,n.sims=NULL, burnin =NULL, thin =NULL,inv_lambda=NULL,var_real_w=NULL,var_real_v=NULL,rho=NULL,N=NULL,T=NULL,c=NULL,c2=NULL, prioravgeff=NULL,acceptlist=NULL,seed=NULL,methodrho=NULL,methodu=NULL) {

## Set the seed
set.seed(seed)

## Create matrices which will store information on simulations ##
inv_lambda.P <<- matrix(NA,nrow=n.repetitions,ncol=1)
hv.P <<- matrix(NA,nrow=n.repetitions,ncol=1)
hw.P <<- matrix(NA,nrow=n.repetitions,ncol=1)
beta.P <<-  matrix(NA,nrow=n.repetitions,ncol=J)
u.P <<-  matrix(NA,nrow=n.repetitions,ncol=N*T)
w.P <<-  matrix(NA,nrow=n.repetitions,ncol=N)
betasd.P <<-  matrix(NA,nrow=n.repetitions,ncol=J)
rho.P  <<-  matrix(NA,nrow=n.repetitions,ncol=1)
sd_inv_lambda.P <<- matrix(NA,nrow=n.repetitions,ncol=1)
sd_hv.P <<- matrix(NA,nrow=n.repetitions,ncol=1)
sd_hw.P <<- matrix(NA,nrow=n.repetitions,ncol=1)
sd_beta.P <<-matrix(NA,nrow=n.repetitions,ncol=J)
sd_u.P <<-matrix(NA,nrow=n.repetitions,ncol=1)
sd_w.P <<-matrix(NA,nrow=n.repetitions,ncol=1)
sd_rho.P <<-matrix(NA,nrow=n.repetitions,ncol=1)
m_inv_lambda.P <<- matrix(NA,nrow=n.repetitions,ncol=1)
m_hv.P <<- matrix(NA,nrow=n.repetitions,ncol=1)
m_hw.P <<- matrix(NA,nrow=n.repetitions,ncol=1)
m_beta.P <<-  matrix(NA,nrow=n.repetitions,ncol=J)
m_u.P <<-  matrix(NA,nrow=n.repetitions,ncol=1)
m_w.P <<-  matrix(NA,nrow=n.repetitions,ncol=1)
m_betasd.P <<-  matrix(NA,nrow=n.repetitions,ncol=J)
m_rho.P<<-  matrix(NA,nrow=n.repetitions,ncol=1)
fake_u.P <<-  matrix(NA,nrow=n.repetitions,ncol=N*T)
sd_fake_u.P <<-  matrix(NA,nrow=n.repetitions,ncol=N*T)
m_fake_u.P <<-  matrix(NA,nrow=n.repetitions,ncol=N*T)
fake_w.P <<-  matrix(NA,nrow=n.repetitions,ncol=N)
sd_fake_w.P <<-  matrix(NA,nrow=n.repetitions,ncol=N)
m_fake_w.P <<-  matrix(NA,nrow=n.repetitions,ncol=N)

mean_accept_u <<-  matrix(NA,nrow=n.repetitions,ncol=N*T)
min_accept_u <<-  matrix(NA,nrow=n.repetitions,ncol=1)
max_accept_u <<-  matrix(NA,nrow=n.repetitions,ncol=1)
mean_accept_rho <<-  matrix(NA,nrow=n.repetitions,ncol=1)
eff_sample_size_Rho <<-  matrix(NA,nrow=n.repetitions,ncol=1)
eff_sample_size_u <<-  matrix(NA,nrow=n.repetitions,ncol=N*T)
inv_lambda_diag <<- matrix(NA,nrow=n.repetitions,ncol=1)
hv_diag <<- matrix(NA,nrow=n.repetitions,ncol=1)
hw_diag <<- matrix(NA,nrow=n.repetitions,ncol=1)
beta_diag <<-  matrix(NA,nrow=n.repetitions,ncol=J)
rho_diag <<- matrix(NA,nrow=n.repetitions,ncol=1)

## Record time spent in simulations
timespent <<- matrix(NA,nrow=n.repetitions,ncol=1)
Throughput_u <<- matrix(NA,nrow=n.repetitions,ncol=1)
Throughput_rho <<- matrix(NA,nrow=n.repetitions,ncol=1)

### WRAPPED FUNCTION STARTS HERE - THIS IS RUN N.REPETITIONS TIMES ###
######################################################################

gibbs_P <- function(n.repetitions=NULL,J=NULL, n.sims=NULL, burnin =NULL, thin =NULL,inv_lambda=NULL,var_real_w=NULL,var_real_v=NULL,rho=NULL,N=NULL,T=NULL,c=NULL,c2=NULL, prioravgeff=NULL,acceptlist=NULL,seed=NULL,methodrho=NULL,methodu=NULL) {

  #### Let us define some Spatial aspects of the model first ####
  rm(list=ls())
  n = N
  data = data.frame(n1=1:n)
  # coords
  data$lat = rep(1:sqrt(n), sqrt(n))
  data$long = sort(rep(1:sqrt(n), sqrt(n)))
  # create W matrix
  wt1 = as.matrix(dist(cbind(data$long, data$lat), method = "euclidean", upper=TRUE))
  wt1 = ifelse(wt1==1, 1, 0)
  diag(wt1) = 0
  # row standardized
  rs = rowSums(wt1)
  NT <- N*T
  wt1 = apply(wt1, 2, function(x) x/rs)
  lw1 = mat2listw(wt1, style="W")
  rhho = rho
  b1 = 1
  b2 = 1
  W = invIrW(lw1, rho=rhho, method="solve", feasible=NULL)
  W <- matrix(W,ncol=N,nrow=N)


  ### INEFFICIENCY - EXPONENTIALLY DISTRIBUTED ###
  fake_su <- matrix(rexp(N*T,rate=inv_lambda))
  ### NOISE - NORMAL, MEAN 0, VARIANCE VAR_REAL_V ###
  fake_sv <- matrix(rnorm(N*T, mean=0, sd=sqrt(var_real_v)))
  ### N FIRMS ###
  fake_sw2 <- matrix(rnorm(N,mean=0,sd=sqrt(var_real_w)))
  ### N*T OBSERVATIONS ###
  fake_sw <- kronecker(fake_sw2,as.matrix(rep(1,T)))


  ### DATA IS A CONSTANT PLUS A NORMAL COVARIATE ###
  X <- cbind(c(1), rnorm(N*T,0,1))
  ### J columns of independent variables
  J <- ncol(X)
  ### Dependent variable where b1=X[,1]
  y <- (b1+(b2*X[,2])+fake_sv+fake_sw)
  lt <- as.matrix(rep(1,T))
  id <- matrix(kronecker(seq(1:N),lt))
  time <- matrix(rep(seq(1:T),N))
  y4 <- data.frame(y,X,id,time,fake_su)
  y5 <- arrange(y4,time,id)
  X <- as.matrix(y5[,2:3])
  y5[,1] <- y5[,1] +  kronecker(diag(T),W)%*%as.matrix(y5[,6])
  ### THEREFORE, EXPECT Beta0=Beta1=1 ###
  y <- as.matrix(y5[,1])
  J <- ncol(X)
  ### Recalling Spatial Weights Matrix ###
  W <- wt1

  ### PRIOR AVERAGE EFFICIENCY AS DEFINED BY THE USER ###
  prioravgeff <-  prioravgeff

  ### Starting values for all variables
  beta.start <- (matrix(rnorm(J,1,0.01)))
  inv_lambda.start <- 5
  hv.start <- 100
  hw.start <- 20
  w.start<- matrix(rnorm(N,mean=0,sd=0.1))
  u.start<-matrix(rnorm(N*T,mean=0.1,sd=0.02))
  rho.start <- rho-0.05
  rho.cur <- rho.start
  
  beta.cur <- beta.start
  inv_lambda.cur <- inv_lambda.start
  hv.cur <- hv.start
  hw.cur <- hw.start
  u.cur <- u.start
  w.cur <- w.start

  ### SETTING MATRIX SIZES TO STORE DRAWS FROM SIMULATIONS FOR ALL VARIABLES ###

  u.draws <- matrix(NA, nrow = ((n.sims-burnin)/thin), ncol = N*T)
  w.draws <- matrix(NA, nrow = ((n.sims-burnin)/thin), ncol = N)
  inv_lambda.draws <- matrix(NA, nrow = ((n.sims-burnin)/thin), ncol = 1)
  hv.draws <- matrix(NA, nrow = ((n.sims-burnin)/thin), ncol = 1)
  hw.draws <- matrix(NA, nrow = ((n.sims-burnin)/thin), ncol = 1)
  beta.draws <- matrix(NA, nrow = ((n.sims-burnin)/thin), ncol = J)
  rho.draws <- matrix(NA, nrow = ((n.sims-burnin)/thin), ncol = 1)
  acceptrho<-matrix(NA, nrow = ((n.sims-burnin)/thin), ncol = 1)
  acceptu<-matrix(NA, nrow = ((n.sims-burnin)/thin), ncol = N*T)
  earlyacceptrho<-matrix(NA, nrow = burnin, ncol = 1)
  earlyacceptu<-matrix(NA, nrow = burnin, ncol = N*T)

  ### SIMULATION FOR N.SIMS BEGINS BELOW - PREPARATION WORK ###
  Winv <- solve((diag(N)-(as.numeric(rho.cur)*W)))
  diagT <- diag(T)
  Winvu <- matrix(solve((diag(N)-as.numeric(rho.cur)*W))%*%matrix(u.cur, ncol=T,nrow=N),nrow=N*T,ncol=1)
  Xinvcrossprod <- solve((crossprod(X)))
  Pn <- matrix(1,nrow=N*T,ncol=1)
  Pnrho <- matrix(1,nrow=1,ncol=1)
  accept_u <- matrix(NA,nrow=N*T,ncol=1)
  accept_rho <- matrix(NA,nrow=1,ncol=1)
  Xbeta <- X%*%beta.cur
  acceptlist <- acceptlist
  logprioreff<- log(prioravgeff,base=exp(1))
  diagN <- diag(N)

  rhofunction <- function(x){(-(crossprod(y-Xbeta-matrix(solve((diagN-as.numeric(x)*W))%*%uaux,nrow=N*T,ncol=1)-rep(w.cur,T))/((2)*(1/hv.cur))))}

  ### LOOP OVER N.SIMS ###
  
  for (i in 1:n.sims) {


    Raux2 <- matrix(rowSums(matrix(matrix(y-Xbeta-Winvu), ncol=T,nrow=N)),ncol=1,nrow=N)
    ### Auxiliary cals for w.cur ###
    mean_w_aux <- as.numeric(((1/hv.cur)*(1/hw.cur))/((T*(1/hw.cur)+(1/hv.cur))*(1/hv.cur)))
    ### w.cur - Normal Distribution ###
    w.cur <- rnorm(N,mean=mean_w_aux*Raux2,sd=sqrt(((1/hv.cur)*(1/hw.cur))/((T*(1/hw.cur)+(1/hv.cur)))))
    ### hw.cur - Gamma Distribution ###
    hw.cur <- matrix(rgamma(1,shape= N/2, rate=0.5*crossprod(w.cur)))
    ### Beta.cur - Normal distribution where "Sigma" is var-covar matrix###
    beta.cur <- matrix(rmvnorm (n=1, mean=(Xinvcrossprod%*%crossprod(X,(y-Winvu-rep(w.cur,T)))),sigma=(as.numeric(1/hv.cur)*(Xinvcrossprod))))
    ### hv.cur - Gamma distribution ###
    Xbeta <- X%*%beta.cur
    hv_aux <- (y-Xbeta-Winvu-rep(w.cur,T))
    hv.cur <- rgamma(1,shape=((N*T)/2),rate=0.5*(crossprod(hv_aux)))
    ### Now draws for inefficiencies ###
    umean <- (y-Xbeta-rep(w.cur,T)-as.numeric(inv_lambda.cur/hv.cur))
    ifelse(methodu==2, u.cand<-u.cur+Pn*c2*c(abs(rnorm(N*T,0,1))) ,u.cand<-u.cur+c2*((rnorm(N*T,0,1))))
    ucandaux <- matrix(u.cand, ncol=T,nrow=N)
    Invaux <- solve((diagN-as.numeric(rho.cur)*W))
    W2 <- matrix((Invaux)%*%ucandaux,nrow=N*T,ncol=1)

    ucurdens <- matrix(exp((-(((Winvu-((umean)))^2))/(2*(1/hv.cur)))-((u.cur-Winvu))*as.numeric(inv_lambda.cur)))
    ucurcand <- matrix(exp((-(((W2-((umean)))^2))/(2*(1/hv.cur)))-((u.cand-W2))*as.numeric(inv_lambda.cur)))
    accept_probability <- matrix(ucurcand/ucurdens)
    uni_draw <- matrix(runif(N*T,0,1))
    accept_probability[u.cand<0]=0
    temp <- u.cur
    accept_u <- ifelse (uni_draw <= accept_probability,accept_u <- 1,accept_u <- 0)
    u.cur <- ifelse (uni_draw <= accept_probability,u.cur <- u.cand, u.cur <- temp)
    Pn <- ifelse (uni_draw <= accept_probability,Pn <- Pn, Pn <- -Pn)

    ### inv.lambda.cur - Gamma Distribution  ###
    inv_lambda.cur <- matrix(rgamma(1, shape=N*T+1,  rate = crossprod((u.cur),matrix(rep(1,N*T)))-logprioreff))
    ## TARGET DENSITY ##
    uaux <- matrix(u.cur, ncol=T,nrow=N)
    Invaux <- solve((diagN-as.numeric(rho.cur)*W))
    Winvu <- matrix((Invaux)%*%uaux,nrow=N*T,ncol=1)
    if(methodrho==3){ rho.cur <- as.numeric(uni.slice(x0=rho.cur,rhofunction,lower=-1, upper=1,w=0.3))
    }
    else{
    rhodensity<- (-(crossprod(y-Xbeta-Winvu-rep(w.cur,T))/((2)*(1/hv.cur))))
    cand_draw <- ifelse(methodrho==2,cand_draw <- rho.cur+Pnrho*c*abs(rnorm(1,0,1)),cand_draw <- rho.cur+c*(rnorm(1,0,1)))
    Invaux2 <- solve((diagN-as.numeric(cand_draw)*W))
    rhocandinv <- matrix((Invaux2)%*%uaux,nrow=N*T,ncol=1)
    rhocand_density<-(-(crossprod(y-Xbeta-rhocandinv-rep(w.cur,T))/((2)*(1/hv.cur))))
    accept_probability2 <- min(1,exp(rhocand_density-rhodensity))
    uni_draw <- runif(1,0,1)
    ###
    ifelse(cand_draw<=-1 | cand_draw>=1, accept_probability2 <- 0, 0)
    ifelse (uni_draw <= accept_probability2,rho.cur <- cand_draw,Pnrho <- -Pnrho)
    ifelse (uni_draw <= accept_probability2,accept_rho<-1,accept_rho<-0)
    }
    Invaux <- solve((diagN-as.numeric(rho.cur)*W))
    Winvu <- matrix(Invaux%*%(uaux),nrow=N*T,ncol=1)

    ### REGISTERING ALL DRAWS IN CORRESPONDING MATRIX ###

    if (i > burnin & (i - burnin)%%thin == 0) {
      beta.draws[(i - burnin)/thin, ]<- beta.cur
      inv_lambda.draws[(i - burnin)/thin,] <- inv_lambda.cur
      hv.draws[(i - burnin)/thin,] <- hv.cur
      u.draws[(i - burnin)/thin, ] <- u.cur
      rho.draws[(i - burnin)/thin,] <- rho.cur
      w.draws[(i - burnin)/thin, ] <- w.cur
      hw.draws[(i - burnin)/thin,] <- hw.cur
      acceptrho[(i - burnin)/thin,]<-accept_rho
      acceptu[(i - burnin)/thin,]<-accept_u
    }

    if (i<burnin){
      earlyacceptu[i,] <- accept_u
      earlyacceptrho[i,] <- accept_rho
    }

    if (i %in% acceptlist){
      
      ### TUNING OF GWM/MH ALGORITHMS ###
      temp2 <- c2
      temp1 <- c
      c2 <- ifelse (matrix(colMeans(earlyacceptu[(i-600):i,]))>matrix(0.50,N*T),c2 <- c2*1.3,c2 <- temp2)
      temp2 <- c2
      c2 <- ifelse (matrix(colMeans(earlyacceptu[(i-600):i,]))<matrix(0.40,N*T),c2 <- c2/1.3,c2<-temp2)
      c <- ifelse ((sum(earlyacceptrho[(i-600):i])/length(earlyacceptrho[(i-600):i]))>0.50,c <- c*1.3,c <-temp1)
      temp1 <- c
      c <- ifelse ((sum(earlyacceptrho[(i-600):i])/length(earlyacceptrho[(i-600):i]))<0.40,c <- c/1.3,c<-temp1)}
    if (i==burnin){
      start.time <<- Sys.time()
    }
  }

  ### RETURN RESULTS ON A LIST ###
  return(list(acceptrho=acceptrho,acceptu=acceptu,fake_su=fake_su,fake_sw2=fake_sw2, rho.draws=rho.draws, beta.draws = beta.draws, inv_lambda.draws=inv_lambda.draws,hv.draws=hv.draws,u.draws=u.draws,w.draws=w.draws,hw.draws=hw.draws,c=c,c2=c2))}

##### FINAL SIMULATION AFTER RUNNING #####

for (k in 1:n.repetitions){

  ####### RUN PROGRAM #######

  Results <- gibbs_P(n.repetitions=NULL,J=NULL,n.sims=NULL, burnin =NULL, thin =NULL,inv_lambda=NULL,var_real_w=NULL,var_real_v=NULL,rho=NULL,N=NULL,T=NULL,c=NULL,c2=NULL, prioravgeff=NULL,acceptlist=NULL,seed=NULL,methodrho=NULL,methodu=NULL)

  ####### CRUCIAL RESULTS #######

  inv_lambda.P[k] <<- colMeans(Results$inv_lambda.draws)
  hv.P[k] <<- colMeans(Results$hv.draws)
  hw.P[k] <<- colMeans(Results$hw.draws)
  beta.P[k,] <<- colMeans(Results$beta.draws)
  u.P[k,] <<- colMeans(Results$u.draws)
  w.P[k,] <<- colMeans(Results$w.draws)
  rho.P[k] <<- colMeans(Results$rho.draws)

  sd_inv_lambda.P[k] <<- sd(Results$inv_lambda.draws)
  sd_hv.P[k] <<- sd(Results$hv.draws)
  sd_hw.P[k] <<- sd(Results$hw.draws)
  sd_u.P[k] <<- sd(Results$u.draws)
  sd_w.P[k] <<- sd(Results$w.draws)
  sd_rho.P[k] <<-sd(Results$rho.draws)
  sd_beta.P[k,] <<-apply(Results$beta.draws, 2, sd)

  m_inv_lambda.P[k] <<- median(Results$inv_lambda.draws)
  m_hv.P[k] <<- median(Results$hv.draws)
  m_hw.P[k] <<- median(Results$hw.draws)
  m_beta.P[k,] <<- colMedians(Results$beta.draws)
  m_u.P[k] <<- median(Results$u.draws)
  m_w.P[k] <<- median(Results$w.draws)
  m_rho.P[k] <<- median(Results$rho.draws)

  fake_u.P[k,] <<- (Results$fake_su)
  sd_fake_u.P[k,] <<- sd(Results$fake_su)

  m_fake_u.P[k,] <<- median(Results$fake_su)
  fake_w.P[k,] <<- (Results$fake_sw2)

  sd_fake_w.P[k,] <<- sd(Results$fake_sw2)
  m_fake_w.P[k,] <<- median(Results$fake_sw2)

  mean_accept_u[k,] <<- colMeans(Results$acceptu)
  mean_accept_rho[k] <<- mean(Results$acceptrho)

  max_accept_u[k] <<- max(colMeans(Results$acceptu))
  min_accept_u[k] <<- min(colMeans(Results$acceptu))

  eff_sample_size_Rho[k] <<- as.numeric(effectiveSize(Results$rho.draws))
  eff_sample_size_u[k,] <<- as.numeric(effectiveSize(Results$u.draws))
  
  ###### CONVERGENCE AND ADDITIONAL INFO ######
  beta_diag[k,] <<-        t(matrix(geweke.diag(Results$beta.draws, frac1=0.1, frac2=0.5)$z))
  rho_diag[k] <<-         t(matrix(geweke.diag(Results$rho.draws, frac1=0.1, frac2=0.5)$z))
  hw_diag[k] <<-          t(matrix(geweke.diag(Results$hw.draws, frac1=0.1, frac2=0.5)$z))
  hv_diag[k] <<-         t(matrix(geweke.diag(Results$hv.draws, frac1=0.1, frac2=0.5)$z))
  inv_lambda_diag[k] <<- t(matrix(geweke.diag(Results$inv_lambda.draws, frac1=0.1, frac2=0.5)$z))

  end.time <- Sys.time()
  time.taken <- difftime(end.time, start.time, units = "secs")
  timespent[k] <<- time.taken
  print(sprintf("Running simulation part %s of %s. Processing time took %f seconds, excluding burnin time.", k,n.repetitions,time.taken))

  Throughput_rho[k] <<- eff_sample_size_Rho[k]/timespent[k]
  Throughput_u[k] <<- mean(eff_sample_size_u[k,])/timespent[k]
}

}


#### EXAMPLE OF HOW TO RUN THE MODEL



###### SCENARIO 1 - N=49 - RHO 0.6 #######

  Simulation_Results <- Simulation(seed=2,methodrho=1,methodu=1,n.repetitions=200,J=2,n.sims=30000, burnin =10000, thin =1,inv_lambda=4,
                                   var_real_w=0.05,var_real_v=0.01,rho=0.6, prioravgeff=0.8,acceptlist=seq(601, 5401, by = 600),
                                   N=49,T=10,c=0.15/10,c2=matrix(0.1,49*10,1))

